module.exports = PanoramaModule = {
    Panorama: {
        // Global variables to store configuration
        iFrameRequestUrl: null,
        panoramaContainer: null,
        panoramaContainerDisplay: null,
        iFrameElement: null,
        panoramaClickCheckBox: null,
        closeButton: null,
        keyboardMovementEnabled: null,
        mapClickEnabled: null,
        markerImage: null,
        // Variable to store whether the panorama is loaded or not
        panoramaLoaded: false,
        // Opens the panorama at the location x,y
        OpenPanoramaOnLocation: function (x, y) {
            // If panorama is closed, open it
            if (PanoramaModule.Panorama.panoramaContainer.style.display === 'none') {
                // Open panorama
                PanoramaModule.Panorama.OpenPanoramaFrame();
            }
            // If panorama is not loaded yet
            if (PanoramaModule.Panorama.panoramaLoaded === false) {
                // Repeat this function until panorama is loaded
                timer = setTimeout(function () {
                    PanoramaModule.Panorama.OpenPanoramaOnLocation(x, y);
                }, 50);
            }
            // If panorama is loaded
            else {
                // If keyboard movement is enabled
                if (PanoramaModule.Panorama.keyboardMovementEnabled === true) {
                    // Send this message once
                    PanoramaModule.Panorama.keyboardMovementEnabled = false;
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ keyboardEnabled: true }, '*');
                }
                // When the frame is loaded post message to open new coordinates
                PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ coordinates: [x, y] }, '*');
            }
        },
        // Opens the panorama iframe
        OpenPanoramaFrame: function () {
            // Fill in the iframes source
            if (PanoramaModule.Panorama.iFrameElement.src !== PanoramaModule.Panorama.iFrameRequestUrl) {
                PanoramaModule.Panorama.iFrameElement.src = PanoramaModule.Panorama.iFrameRequestUrl;
            }
            // Unhide it
            PanoramaModule.Panorama.panoramaContainer.style.display = '';
            // Add event listener to close button
            PanoramaModule.Panorama.closeButton.addEventListener('click', PanoramaModule.Panorama.HidePanoramaFrame);
            // Get the marker
            const marker = document.getElementById('panorama-view-cone');
            // If there is a marker
            if (marker !== null) {
                // Enable marker
                marker.style.display = '';
            }
        },
        // Hides the panorama frame
        HidePanoramaFrame: function (event) {
            // Hide container
            PanoramaModule.Panorama.panoramaContainer.style.display = 'none';
            // Get the marker
            const marker = document.getElementById('panorama-view-cone');
            // If there is a marker
            if (marker !== null) {
                // Remove marker
                marker.style.display = 'none';
            }
        },
        // Configure MapLibre map for panorama usage, call this method in map.onload
        ConfigureMapLibreMap: function (map, maplibregl) {
            // If keyboard movement is on
            if (PanoramaModule.Panorama.keyboardMovementEnabled === true) {
                // Disable map keyboard controls
                map.keyboard.disable();
            }
            // If map is clickable
            if (PanoramaModule.Panorama.mapClickEnabled === true) {
                // Open panorama on click location
                map.on('click', function (e) {
                    // If checkbox is checked
                    if (PanoramaModule.Panorama.panoramaClickCheckBox.checked === true) {
                        // Open panorama on location
                        PanoramaModule.Panorama.OpenPanoramaOnLocation(e.lngLat.lng, e.lngLat.lat);
                    }
                })
            }
            // If marker is enabled (exists)
            if (PanoramaModule.Panorama.markerImage !== null) {
                // Create new marker
                const newMarker = document.createElement('div');
                newMarker.id = 'panorama-view-cone';
                const img = new Image();
                img.addEventListener("load", function () {
                    newMarker.style.width = img.naturalWidth + 'px';
                    newMarker.style.height = img.naturalHeight + 'px';
                    newMarker.style.backgroundImage = 'url(' + img.src + ')';
                });
                img.src = PanoramaModule.Panorama.markerImage;
                //newMarker.style.backgroundImage = "url(" + panorama.markerImage + ")";
                let maplibreMarker = new maplibregl.Marker(newMarker);
                // When new location or angle comes as message
                window.addEventListener('message', function (event) {
                    if (event.data.angle !== undefined && event.data.coordinates !== undefined) {
                        // If panorama is open
                        if (PanoramaModule.Panorama.panoramaContainer.style.display !== 'none') {
                            // Get the marker
                            const marker = document.getElementById('panorama-view-cone');
                            // If there is a marker
                            if (marker !== null) {
                                // Get its coordinates
                                const markerCoordinates = maplibreMarker.getLngLat();
                                // If the new coordinates are different than the markers coordinates
                                if (markerCoordinates !== null && markerCoordinates.lng !== parseFloat(event.data.coordinates[0]) && markerCoordinates.lat !== parseFloat(event.data.coordinates[1])) {
                                    // Center map
                                    map.flyTo({ center: event.data.coordinates });
                                }
                            }
                            else {
                                maplibreMarker.setLngLat([29, 41]).addTo(map);
                            }
                            maplibreMarker.setLngLat(event.data.coordinates).setRotation(event.data.angle);
                        }
                    }
                })
            }
        },
        // Configure Panorama iFrameId is html id of the iframe, close button id is the html id of the close button, if marker is rendered on map markerId is the html id of the marker
        /**
         * @param {String} properties.token The request token
         * @param {String} properties.panoramaType Default: 2013-2018 - The type of panorama, empty string for 2013-2018, 2018 for 2018 and 2013 for 2013
         * @param {String} properties.panoramaContainerId Default: panorama-container - The container that includes the panorama and its related controls
         * @param {String} properties.panoramaIframeId Default: panorama-iframe - Id of the iframe element
         * @param {String} properties.panoramaCloseButtonId Default: panorama-close-button - Id of the button that closes the panorama container
         * @param {Boolean} properties.mapClickEnabled Default: false - The parameter that decides whether a click on the map will open the panorama, loads default checkbox with id: "panorama-checkbox"
         * @param {String} properties.panoramaCheckBoxElementId The checkbox that enables "click map open panorama".
         * @param {String} properties.markerImage Default: https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png - The image for the view cone
         * @param {Boolean} properties.keyboardControlsEnabled Default: true - Enables or disables keyboard movement inside the panorama.
        */
        ConfigurePanorama: function (properties) {
            // Initialize global variables
            if (properties.panoramaType === null || properties.panoramaType === undefined) {
                properties.panoramaType = '';
            }
            PanoramaModule.Panorama.iFrameRequestUrl = 'https://cbspanorama.ibb.gov.tr/panorama/panorama' + properties.panoramaType + '/?token=' + properties.token;
            // Optional, default is panorama-container
            if (properties.panoramaContainerId !== null && properties.panoramaContainerId !== undefined) {
                PanoramaModule.Panorama.panoramaContainer = document.getElementById(properties.panoramaContainerId);
            }
            else {
                PanoramaModule.Panorama.panoramaContainer = document.getElementById('panorama-container');
            }
            // Get the panorama containers display
            PanoramaModule.Panorama.panoramaContainerDisplay = window.getComputedStyle(PanoramaModule.Panorama.panoramaContainer).display;
            PanoramaModule.Panorama.panoramaContainer.style.display = 'none';
            // Optional, default is panorama-iframe
            if (properties.panoramaIframeId !== null && properties.panoramaIframeId !== undefined) {
                PanoramaModule.Panorama.iFrameElement = document.getElementById(properties.panoramaIframeId);
            }
            else {
                PanoramaModule.Panorama.iFrameElement = document.getElementById('panorama-iframe');
            }
            // Optional, default is panorama-close-button
            if (properties.panoramaCloseButtonId !== null && properties.panoramaCloseButtonId !== undefined) {
                PanoramaModule.Panorama.closeButton = document.getElementById(properties.panoramaCloseButtonId);
            }
            else {
                PanoramaModule.Panorama.closeButton = document.getElementById('panorama-close-button');
            }
            // Optional, default is false
            if (properties.mapClickEnabled === true) {
                PanoramaModule.Panorama.mapClickEnabled = properties.mapClickEnabled;
                PanoramaModule.Panorama.markerImage = 'https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png';
                PanoramaModule.Panorama.panoramaClickCheckBox = document.getElementById('panorama-checkbox');
            }
            else {
                PanoramaModule.Panorama.mapClickEnabled = false;
            }
            // Optional, default is none element, false mapclick
            if (properties.panoramaCheckBoxElementId !== null && properties.panoramaCheckBoxElementId !== undefined) {
                PanoramaModule.Panorama.panoramaClickCheckBox = document.getElementById(properties.panoramaCheckBoxElementId);
                PanoramaModule.Panorama.mapClickEnabled = true;
            }
            // Optional, default is 'https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png'
            if (properties.markerImage !== null && properties.markerImage !== undefined) {
                PanoramaModule.Panorama.markerImage = properties.markerImage;
            }
            else {
                PanoramaModule.Panorama.markerImage = 'https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png';
            }
            // Optional, default is true
            if (properties.keyboardControlsEnabled !== null && properties.keyboardControlsEnabled !== undefined) {
                PanoramaModule.Panorama.keyboardMovementEnabled = properties.keyboardControlsEnabled;
            }
            else {
                PanoramaModule.Panorama.keyboardMovementEnabled = true;
            }
            // Catch the panorama loaded message
            window.addEventListener('message', function (event) {
                if (event.data.load === true) {
                    PanoramaModule.Panorama.panoramaLoaded = true;
                }
            });
            // If configuration allows keyboard movement
            if (PanoramaModule.Panorama.keyboardMovementEnabled === true) {
                PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ keyboardEnabled: true }, '*');
            }
        }
    }
}